$ontext
GAMS version of the optimal scheduling of multiproduct pipeline
Reference: Diego Cafaro and Jaime Cerda, "Optimal scheduling of
multiproduct pipeline systems using a non-discrete MILP formulation"
Computers and Chemical Engineering 28 (2004) 2053-2068
$offtext

SETS
i       set of chronologically ordered pumping runs /i1*i9/
iold(i) set of old pumping runs inside the pipeline at the start of the time horizon /i1*i5/
inew(i) set of new pumping runs to be potentially executed during the time horizon/i6*i9/
S       set of derivative oil products /p1*p4/
J       set of depots along the pipeline /d1*d5/
O       set of scheduled production runs in the refinery during the time horizon /o1*o4/
K       set of high cost intervals /k1,k2/
asigna(o,s) /o1.p1,o2.p2,o3.p3,o4.p4/
t       disjunction terms for high interval cost constraints /1*4/ ;

Alias(ss,s);
ALIAS(ii,i);
Alias(jj,j);
Alias(inew2,inew);

Scalars
e                                /0.01/
vbmin    minimum pumping rate    /10/
vbmax    maximum pumping rate    /10/
M                                /2000/
MT                               /80/
lmin minimum pumping run lenght  /.5/
lmax maximum pumping run lenght  /50/
Dmax maximum derivation          /1500/
va   /5/
vm maximum flow rate from depots to the market /7/
nd /5/
CEP costo extra /20/
hmax    time horizon length /75/
;

Parameters

P(j) longitud del ducto /d1 100, d2 200, d3 300, d4 400, d5 475/;


parameters
qp(o) cantidad de producto recibido /o1 250, o2 250, o3 125, o4 125/
a(o)  tiempo de arribo de la ordenes de produccion /o1 0, o2  0, o3  50, o4 50/
b(o)  /o1 50, o2 50, o3 75, o4 75/

CF(I)  /i1 475, i2 400, i3 375, i4 250, i5 75/

IHP(k) /k1 15, k2 40/
FHP(k) /k1 25, k2 50/

CmantR(s) /p1 2, p2 2.3, p3 7, p4 2.5/
IIR(s) /p1 500, p2 520, p3 210, p4 515/
IRmin(s) /p1 270, p2 270, p3 50, p4 270/
IRmax(s) /p1 1200, p2 1200, p3 350, p4 1200/
CmantD(s) /p1 10, p2 15.5, p3 20, p4 17/
;

Table qd(j,s) supply demand
    p1    p2    p3    p4
d1  100   70    60    60
d2  110   90    40    50
d3  120   100   40    50
d4  120   80    0     50
d5  150   100   20    50;

Table tau(s,ss) change overs on pumping
   p1 p2 p3 p4
p1 0  0  0  0
p2 0  0  0  0
p3 0  0  0  0
p4 0  0  0  0;


Table IFS(s,ss) interfase
    p1   p2   p3   p4
p1  0    .3   .37  .35
p2  .3   0    100  .38
p3  .37  100  0    100
p4  .35  .38  100  0;

Table IID(s,j)
    d1   d2   d3   d4   d5
p1  190  230  200  240  190
p2  180  210  180  180  180
p3   50   65   60   60   60
p4  120  140  190  190  170;

Table IDmin(s,j)
    d1  d2  d3  d4  d5
p1  90  90  90  90  90
p2  90  90  90  90  90
p3  10  10  10  10  10
p4  90  90  90  90  90;

Table IDmax(s,j)
    d1   d2   d3   d4   d5
p1  400  400  400  400  400
p2  400  400  400  400  400
p3   70   70   70   70   70
p4  400  400  400  400  400;

Table CNP(s,j)
    d1   d2   d3   d4   d5
p1  3.5  4.5  5.5  6.0  6.9
p2  3.6  4.6  5.6  6.2  7.3
p3  4.8  5.7  6.8  7.9  8.9
p4  3.7  4.7  5.7  6.1  7.0;

BINARY VARIABLES
u(i,k)       high cost time interval Kwh
y(i,s)       denoting that product s is contained in run i
x(i,j,ii)    denoting that some portion of run i is transferred to depot j
zu(i,o)      denoting that run i ends after the begining of refinery production run o
zl(i,o)      denoting that run i begins after the end of refinery production run o

v(i,k)
wi(i,k,t)
ya(i,s) is equal to y but needed because of the disjunction definitions;


POSITIVE VARIABLES
C(i)            completion time of run i
L(i)            length of run i
H(i,k)          usage of high cost period in run i
F(i,ii)         upper volumetric coordinate of run i at  time Ci'
W(i,ii)         volume of run i at time Ci'
Q(i)            volume of product injected in the pipeline while pumping run i
PP(i,s)         volume of product s injected in the pipeline while pumping run i
D(i,j,ii)       volume of run i draw off the pipeline to depot j while pumping run
DV(i,s,j,ii)    volume of product s transferred from run i to depot j while pumping run i'
WIF(i)          interface volume between run i  and run (i-1)
ID(s,j,ii)      inventory of product s in depot j at the completion time of run i'
IRU(s,ii)       inventory of product s in the refinery at the starting time of run
IRL(s,ii)       inventory of product s in the refinery at the completion time of run i'
ql(o,ii)        volume of product available from production run o before starting run i'
qu(o,ii)        volume of product available from production run o before ending run i'
qm(s,j,ii)
TT(i,s,ss)
;


Variable z;

      y.fx('i1','p2')=1;
      y.fx('i2','p1')=1;
      y.fx('i3','p2')=1;
      y.fx('i4','p1')=1;
      y.fx('i5','p2')=1;

EQUATIONS
         FO
         R1(i,s,ss)
         R2(inew),R3(inew),R4(inew),R4b(inew)
         R5(i),
         R6a(i,k),R6b(i,k),R6c(i,k),R6d(i,k)
         R7a(i,k),R7b(i,k),R7c(i,k),R7d(i,k)
         R8(inew),R8b(inew)
         R9(i,s,ss),
         R10(i,ii),R11(i),R11b(i), R12(i,ii),
         R13a(i,j,ii),
         R13b(i,j,ii),
         R14a(i,j,ii),
         R14b(i,j,ii),
         R15(i,j,ii),
         R16(i,ii),R17(ii)
         R18a(i,j),R18b(i),R18c(i,ii),R18d(i,ii),R18e(i,ii),R18f(i,ii)
         R19(i),R20(s,j),R21(s,j,ii)
         R22a(i,o),R22b(i,o),R22c(i,o),R22d(i,o),R22e(i,o)
         R23a(o,i),R23b(o,i),
         R23c(o,i),
         R23d(o,i),
         R24a(o,i),
         R24b(o,i),
         R24c(o,i),
         R25a(i,s),
         R25b(i,s),
         R25c(inew),
         R26a(i,s),R26b(i,s),R26c(i,s),R26d(i,s)
         R27a(i,ii,s,j),
         R27b(i,ii,s,j),
         R27c(i,ii,j),
         R28a(j,s,ii),R28b(j,s,ii),R28c(j,s,inew),R28d(j,s,inew)
         R29(i,ii)
         FICT1,
         INT(i,s)
;


FO..z =e= sum((i,j,s),CNP(s,j)*(sum(inew,DV(i,s,j,inew))))
                 + CEP*vbmax*(sum((i,k),H(i,k)))
                     + 100 * (sum(i,WIF(i)))
                        + sum(s,CmantR(s)/(card(Inew))*(sum(inew,(IRU(s,inew))))
                          + CmantD(s)/(card(Inew))*(sum(inew,(sum(j,ID(s,j,inew))))));

R1(i,s,ss)$(ord(i) gt 5)..C(i)-L(i) =g= C(i-1);
R2(inew)..C(inew) =g= L(inew);
R3(inew)..C(inew) =l= hmax;
R4(inew)..L(inew) =l= lmax*(sum(s,y(inew,s)));
R4b(inew)..L(inew) =g= lmin*(sum(s,y(inew,s)));
R5(i)$(ord(i) gt 5)..sum(s,y(i,s)) =l= sum(s,y(i-1,s));
*
* Constraints for the High Energy Interval Cost
*
R6a(i,k).. C(i)-L(i) =g= IHP(k);
R6b(i,k).. C(i)-L(i) =l= IHP(k)- e;
R6c(i,k).. C(i) =g= 0;
R6d(i,k).. C(i) =g= FHP(k);
R7a(i,k).. H(i,k) =g= L(i);
R7b(i,k).. H(i,k) =g= FHP(k)-IHP(k);
R7c(i,k).. H(i,k) =g= C(i) - IHP(k) ;
R7d(i,k).. H(i,k) =g= FHP(k) - (C(i)-L(i)) ;

R8(inew)..L(inew)*vbmin =l= Q(inew);
R8b(inew)..L(inew)*vbmax =g= Q(inew);
R9(i, s,ss)$(ord(i) gt 1).. WIF(i) =g= IFS(s,ss)*(y(i,ss) + y(i-1, s)-1);
R10(i,ii)$(ord(ii) ge 5 and ord(ii) ge ord(i) and ord(i) gt 1)..
                                 F(i-1,ii)-W(i-1,ii) =e= F(i,ii);
R11(i)$(ord(i) gt 5).. F(i,i) =e= Q(i) - sum(j,D(i,j,i));
R11b(i)$(ord(i) gt 5).. F(i,i) =e= W(i,i);
R12(i,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..W(i,ii) =e= W(i,ii-1)-
                                                 sum(j,D(i,j,ii));
R13a(i,j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..D(i,j,ii) =l= Dmax;
R13b(i,j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..D(i,j,ii) =l= 0.;
R14a(i,j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..F(i,ii) - WIF(i) =g= P(j);
R14b(i,j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..F(i,ii) - WIF(i) =g= 0.;
R15 (i,j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5)..D(i,j,ii) =l= P(j) -
         (F(i,ii-1)-W(i,ii-1)) - sum(jj$(ord(jj) lt ord(j)),D(i,jj,ii));
R16(i,ii)$(ord(ii) gt ord(i) and ord(ii) gt 5)..
                         sum(j,D(i,j,ii)) =l= W(i,ii-1) - WIF(i);

R17(ii)$(ord(ii) ge 5).. sum((i,j)$(ord(i) le ord(ii)),D(i,j,ii)) + F('i1',ii)
                                        - P('d5') =e= Q(ii);

R18a(i,j).. sum(ii$(ord(ii) ge ord(i) and ord(ii) gt 5),D(i,j,ii)) =l=
                  Q(i) - sum((jj,ii)$((ord(jj) lt ord(j))
                                $(ord(ii) ge ord(i)and ord(ii) gt 5 )), D(i,jj,ii));
R18b(i).. Q(i) =g= sum((j,ii)$(ord(ii) ge ord(i) and ord(ii) gt 5),D(i,j,ii));

R18c(i,ii)$(ord(ii) gt ord(i)and ord(ii) gt 5).. F(i,ii) =g= F(i,ii-1);
R18d(i,ii)$(ord(ii) gt ord(i)and ord(ii) gt 5).. F(i,ii)-W(i,ii) =g= F(i,ii-1)-W(i,ii-1);
R18e(i,ii)$(ord(ii) gt ord(i)and ord(ii) gt 5).. F(i,ii) =l= P('d5')+1;
R18f(i,ii)$(ord(ii) ge ord(i)and ord(ii) gt 5).. F(i,ii) =g= W(i,ii);

R19(i).. sum(s,y(i,s)) =l= 1;

R20(s,j).. sum(inew,qm(s,j,inew)) =e= qd(j,s);
R21(s,j,ii)$( ord(ii) gt 5)..
                         qm(s,j,ii) =l= (C(ii)-C(ii-1))*vm;


R22a(i,o)$(ord(i) gt 5)..C(i) =g= a(o);
R22b(i,o)$(ord(i) gt 5)..C(i) =l= a(o)-e;
R22c(i,o)$(ord(i) gt 5)..C(i)-L(i) =g= b(o);
R22d(i,o)$(ord(i) gt 5)..C(i)-L(i) =l= b(o)-e ;
R22e(inew,o)..zu(inew,o) =g= zl(inew,o);


R23a(o,i)$(ord(i) gt 5).. qu(o,i) =l= qp(o);
R23b(o,i)$(ord(i) gt 5).. qu(o,i) =e= 0;
R23c(o,i)$(ord(i) gt 5).. qu(o,i) =l= (C(i)-a(o))*va;
R23d(o,i)$(ord(i) gt 5).. qu(o,i) =l= C(i)*va;

R24a(o,i)$(ord(i) gt 5).. ql(o,i) =g= qp(o);
R24b(o,i)$(ord(i) gt 5).. ql(o,i) =g= 0;
R24c(o,i)$(ord(i) gt 5).. ql(o,i) =g= (C(i)-L(i)-a(o))*va ;


R25a(i,s)$(ord(i) gt 5).. PP(i,s) =l= M;
R25b(i,s)$(ord(i) gt 5).. PP(i,s) =e= 0;
R25c(inew).. sum(s,PP(inew,s)) =e= Q(inew);

R26a(i,s)$(ord(i) gt 5)..IRL(s,i) =e= IIR(s) + sum(o$(asigna(o,s)),qu(o,i))
                 - (sum(ii$(ord(ii) gt 5 and ord(ii) le ord(i)),PP(ii,s)));

R26b(i,s)$(ord(i) gt 5)..IRL(s,i) =g= IRmin(s);
R26c(i,s)$(ord(i) gt 5)..IRU(s,i) =e= IIR(s) + sum(o$(asigna(o,s)),ql(o,i))
                 - (sum(ii$(ord(ii) gt 5 and ord(ii) lt ord(i)),PP(ii,s)));
R26d(i,s)$(ord(i) gt 5)..IRU(s,i) =l= IRmax(s);


R27a(i,ii,s,j)$(ord(ii) ge ord(i) and ord(ii) gt 5)..DV(i,s,j,ii) =l= Dmax;
R27b(i,ii,s,j)$(ord(ii) ge ord(i) and ord(ii) gt 5)..DV(i,s,j,ii) =l= 0;
R27c(i,ii,j)$(ord(ii) ge ord(i) and ord(ii) gt 5)..sum(s,DV(i,s,j,ii)) =e= D(i,j,ii);

R28a(j,s,ii)$(ord(ii) eq 6).. ID(s,j,ii) =e= IID(s,j)
                 + sum(i$(ord(i) le ord(ii)), DV(i,s,j,ii)) - qm(s,j,ii);
R28b(j,s,ii)$(ord(ii) gt 6).. ID(s,j,ii) =e= ID(s,j,ii-1)
                 + sum(i$(ord(i) le ord(ii)), DV(i,s,j,ii)) - qm(s,j,ii);
R28c(j,s,inew).. ID(s,j,inew) =g= IDmin(s,j);
R28d(j,s,inew).. ID(s,j,inew) =l= IDmax(s,j);


R29(i,ii)$(ord(i) le 5 and ord(ii) eq 5)..F(i,ii) =e= CF(i);

FICT1.. sum(i,sum(k, u(i,k)+v(i,k)))
       +sum(i,sum(j, sum(ii$(ord(ii) gt 5),x(i,j,ii))))
       + sum(i,sum(k,sum(t, wi(i,k, t))))    =g= 0;
INT(i,s)$(ord(i) gt 5).. ya(i,s)=e=y(i,s);

* BOUNDS
C.UP(i)          =750.;
L.UP(i)          =750.;
H.UP(i,k)        =750.;
F.UP(i,ii)       =750.;
W.UP(i,ii)       =750.;
Q.UP(i)          =750.;
PP.UP(i,s)       =750.;
D.UP(i,j,ii)     =750.;
DV.UP(i,s,j,ii)  =750.;
WIF.UP(i)        =120.;
ID.UP(s,j,ii)    =400.;
ID.LO(s,j,ii)    =0.;
IRU.UP(s,ii)     =2000.;
IRL.UP(s,ii)     =2000.;
ql.UP(o,ii)      =750.;
qu.UP(o,ii)      = 750.;
qm.UP(s,j,ii)    =750.;


$ONECHO > "%lm.info%"
Disjunction
     D1(i,k), D2(i,k), D3(i,k), D4(i,j,ii), D5(i,o), D6(i,o),D7(i,s),
     D8(i,s);

D8(i,s) is
IF y(i,s) THEN
        R27a(i,ii,s,j) with (ord(ii) > 5 and ord(ii) >= ord(i)
                             and ord(j) >= 1);
ELSE
        R27b(i,ii,s,j)with (ord(ii) > 5 and ord(ii) >= ord(i)
                            and ord(j) >= 1);
ENDIF;


D1(i,k) is
IF u(i,k) THEN
         R6a(i,k);
ELSE
         R6b(i,k);
ENDIF;

D2(i,k) is
IF v(i,k) THEN
         R6c(i,k);
ELSE
         R6d(i,k);
ENDIF;

D3(i,k) is
IF wi(i,k,'1') THEN
         R7a(i,k);
ELSIF wi(i,k,'2') THEN
         R7b(i,k);
ELSIF wi(i,k,'3') THEN
         R7c(i,k);
ELSIF wi(I,k,'4') THEN
         R7d(i,k);
ENDIF;

D4(i,j,ii) with (ord(ii) >= ord(i) and ord(ii) > 5) is
IF x(i,j,ii) THEN
       R13a(i,j,ii);
       R14a(i,j,ii);
       R15(i,j,ii);
ELSE
       R13b(i,j,ii);
       R14b(i,j,ii);
ENDIF;

D5(i,o) with (ord(i) > 5) is
IF zu(i,o) THEN
         R22a(i,o);
         R23a(o,i);
         R23c(o,i);
ELSE
         R22b(i,o);
         R23b(o,i);
         R23d(o,i);
ENDIF;

D6(i,o) with (ord(i) > 5) is
IF zl(i,o) THEN
         R22c(i,o);
         R24a(o,i);
ELSE
         R22d(i,o);
         R24b(o,i);
         R24c(o,i);
ENDIF;

D7(i,s) with (ord(i) > 5) is
IF ya(i,s) THEN
        R25a(i,s);
ELSE
        R25b(i,s);
ENDIF;

$offecho

option mip=lmbigm;
OPtion optcr=0.0;
OPTION OPTCA=0.0;
option reslim=10000.;
Model PROB1 /all/;
Solve PROB1 using MIP minimizing z;
Display y.l,x.l,F.l,L.l,  PP.l,D.l, DV.l, H.l, WIF.l, IRU.l, ID.l, w.l;
