$TITLE MULTIPERIOD DISJUNCTIVE FORMULATION TO MILP (3-process problem)
$OFFSYMXREF
$OFFSYMLIST
* Retrofit problem solved by Jennifer Jackson
* Reference:  Jackson J. and Grossmann I.E.,
*        "High Level Optimization Model for the retrofit planning of
*        process networks", I&EC Research 41,3762-3770 (2002).
*
* FORMULATION WITH OPERATION AND DESIGN VARIABLES
* ALL MODIFICATIONS ARE DISCRETE CHANGES

SET     K  COMPONENTS   /A ACETYLENE
                         B HCN
                         C PROPYLENE
                         D ACETALDEHYDE
                         E ACRYLONITRILE/
        I  UNITS        /1 PROCESS 1
                         2 PROCESS 2
                         3 PROCESS 3/
        J  STREAMS      /1*13/
        M  DISJUNCTIONS TERMS   /0*3/
        N  OPERATIONS   /1*2/
        T  TIME PERIODS /1*3/;

SCALAR
RAW1    /550/
RAW2    /36/
RAW3    /333/;

PARAMETERS
DEM1(T) DEMAND OF D     /1  75, 2  75,  3  75/
DEM2(T) DEMAND OF E     /1 100, 2 100,  3 100/
LC(T)   CAPITAL LIMIT   /1 450, 2 450,  3 450/ ;

TABLE ETA(I,N)
        1       2
1       .9      .95
2       .85     .9
3       .8      .85;

TABLE CAP(I,N)
        1       2
1       50      75
2       85      105
3       65      80;

TABLE P(K,T)    PRICE OF COMPONENT
        1       2       3
A       -20     -20     -20
B       -100    -100    -100
C       -100    -100    -100
D       700     700     700
E       500     500     500 ;


TABLE FC(I,M,T) FIXED COST OF MODIFICATION
        1       2       3
1.0     0       0       0
2.0     0       0       0
3.0     0       0       0
1.1     100     75      60
2.1     100     75      60
3.1     100     75      60
1.2     350     225     250
2.2     350     225     250
3.2     350     225     250
1.3     450     300     310
2.3     450     300     310
3.3     450     300     310;
VARIABLES NPV;

POSITIVE VARIABLES
F(J,T)          MOLAR FLOW RATES
C(I,T)          COST OF MODIFICATIONS
CM(I,M,T)
PRICE(T)        PRICES OF COMPONENTS;

BINARY VARIABLES
Y(I,M,T)  OPERATING VARIABLES;

Binary VARIABLES
W(I,M,T)  DESIGN VARIABLES;

EQUATIONS
N1(T),N2(T),N3(T),N4(T),N5(T),N6(T)
D1(T),D2(T),D3(T),D4(T),D5(T)

L1(I,T),L2(I,M),L3(I),L4(I),L5(I),L6(I),L7(I),L8(I),L9(I),L10(I)
L11(I),L12(I,M),L13(I,M),L14(I,M)
P11(T),P21(T),P31(T),P12(T),P22(T),P32(T),P33(T),P34(T)
CAP11(T),CAP21(T),CAP31(T),CAP12(T),CAP22(T),CAP32(T)
COST1(I,M,T),  COST2(I,T), COST4(T)

PR(T),OBJ;

*NODE BALANCES
N1(T)..        F('1',T) =E= F('2',T)+F('3',T);
N2(T)..        F('4',T)+F('8',T) =E= F('5',T);
N3(T)..        F('6',T) =E= F('7',T);
N4(T)..        F('9',T) =E= F('12',T);
N5(T)..        F('10',T)+F('11',T) =E= F('13',T);
N6(T)..        F('3',T) =E= 10*F('5',T);

*PRODUCT DEMANDS AND SUPPLY LIMITATIONS
D1(T)..        F('12',T) =L= DEM1(T);
D2(T)..        F('13',T) =L= DEM2(T);
D3(T)..        F('1',T) =L= RAW1;
D4(T)..        F('4',T) =L= RAW2;
D5(T)..        F('6',T) =L= RAW3;

*LOGICAL CONSTRAINTS
L1(I,T)..       SUM(M,Y(I,M,T)) =E= 1;
L2(I,M)..       SUM(T,W(I,M,T)) =L= 1;
L3(I)..                Y(I,'1','1') =L= Y(I,'1','2');
L4(I)..                Y(I,'1','1') =L= Y(I,'1','3');
L5(I)..                Y(I,'1','2') =L= Y(I,'1','3');
L6(I)..                Y(I,'2','1') =L= Y(I,'2','2');
L7(I)..                Y(I,'2','1') =L= Y(I,'2','3');
L8(I)..                Y(I,'2','2') =L= Y(I,'2','3');
L9(I)..                Y(I,'3','1') =L= Y(I,'3','2');
L10(I)..        Y(I,'3','1') =L= Y(I,'3','3');
L11(I)..        Y(I,'3','2') =L= Y(I,'3','3');
L12(I,M)..      Y(I,M,'1') =L= W(I,M,'1');
L13(I,M)..      Y(I,M,'2') =L= Y(I,M,'1')+W(I,M,'2');
L14(I,M)..      Y(I,M,'3') =L= Y(I,M,'1')+Y(I,M,'2')+W(I,M,'3');

*PROCESS BALANCES
P11(T)..       F('9',T) =E= ETA('1','1')*F('2',T);
P12(T)..       F('9',T) =E= ETA('1','2')*F('2',T);
P21(T)..       F('10',T) =E= ETA('2','1')*F('5',T);
P22(T)..       F('10',T) =E= ETA('2','2')*F('5',T);
P31(T)..       F('8',T) =E= 0.17*ETA('3','1')*F('7',T);
P32(T)..       F('8',T) =E= 0.17*ETA('3','2')*F('7',T);
P33(T)..       F('11',T) =E= 0.83*ETA('3','1')*F('7',T);
P34(T)..       F('11',T) =E= 0.83*ETA('3','2')*F('7',T);

*CAPACITY LIMITATIONS
CAP11(T)..   F('9',T)  =L= CAP('1','1');
CAP12(T)..   F('9',T)  =L= CAP('1','2');
CAP21(T)..   F('10',T) =L= CAP('2','1');
CAP22(T)..   F('10',T) =L= CAP('2','2');
CAP31(T)..   F('11',T) =L= CAP('3','1');
CAP32(T)..   F('11',T) =L= CAP('3','2');

*MODIFICATION COSTS
COST1(I,M,T)..  CM(I,M,T) =E= FC(I,M,T);
COST2(I,T)..    C(I,T)    =E= sum(M,CM(I,M,T));
COST4(T)..      SUM(I,C(I,T)) =L= LC(T);

PR(T)..        PRICE(T) =E= P('A',T)*F('1',T)+P('B',T)*F('4',T)
                +P('C',T)*F('6',T)+P('D',T)*F('12',T)
                +P('E',T)*F('13',T);

OBJ..   NPV =E= SUM(T, 365*PRICE(T)-SUM(I, 1000*C(I,T)));
$ONECHO > "%lm.info%"
disjunction dis1(T),dis2(T),dis3(T),dis4(T),dis5(T),dis6(T);

 dis1(T) is
   if Y('1','0',T) then
         P11(T);
         CAP11(T);
   elsif Y('1','1',T) then
         P12(T);
         CAP11(T);
   elsif Y('1','2',T) then
         P11(T);
         CAP12(T);
   elsif Y('1','3',T) then
         P12(T);
         CAP12(T);
   endif;

 dis2(T) is
   if Y('2','0',T) then
         P21(T);
         CAP21(T);
   elsif Y('2','1',T) then
         P22(T);
         CAP21(T);
   elsif Y('2','2',T) then
         P21(T);
         CAP22(T);
   elsif Y('2','3',T) then
         P22(T);
         CAP22(T);
   endif;

dis3(T) is
   if Y('3','0',T) then
         P31(T);
         P33(T);
         CAP31(T);
   elsif Y('3','1',T) then
         P32(T);
         P34(T);
         CAP31(T);
   elsif Y('3','2',T) then
         P31(T);
         P33(T);
         CAP32(T);
   elsif Y('3','3',T) then
         P32(T);
         P34(T);
         CAP32(T);
   endif;

dis4(T) is
   if W('1','0',T) then
         COST1('1','0',T);
   elsif W('1','1',T) then
         COST1('1','1',T);
   elsif W('1','2',T) then
         COST1('1','2',T);
   elsif W('1','3',T) then
         COST1('1','3',T);
   endif;

 dis5(T) is
   if W('2','0',T) then
         COST1('2','0',T);
   elsif W('2','1',T) then
         COST1('2','1',T);
   elsif W('2','2',T) then
         COST1('2','2',T);
   elsif W('2','3',T) then
         COST1('2','3',T);
   endif;

dis6(T) is
   if W('3','0',T) then
         COST1('3','0',T);
   elsif W('3','1',T) then
         COST1('3','1',T);
   elsif W('3','2',T) then
         COST1('3','2',T);
   elsif W('3','3',T) then
         COST1('3','3',T);
   endif;
$offecho

F.LO(j,t)=0;
F.UP(j,T)=1000;
C.lo(i,T)=0;
C.up(i,t)=1000;
cm.lo(I,M,t) =0.;
cm.up(i,m,t)=1000.;
MODEL FAKE /ALL/;
FAKE.OPTFILE = 1;

OPTION LIMROW = 0;
OPTION LIMCOL = 0;
OPTION OPTCR = 0;
option OPTCA =0;
OPTION MIP=lmbigm;

SOLVE FAKE USING MIP MAXIMIZING NPV;
